# coding=utf-8
"""
 :Description:    应用类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""


class App(object):
    def __init__(self, package):
        self.PACKAGE = package



# coding=utf-8

import time
import subprocess
import threading

"""
 :Deadb_commandion:    录屏操作
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""


class ScreenRecord(threading.Thread):
    def __init__(self, device):
        threading.Thread.__init__(self)
        self._DEVICE = device
        self._SIZE = None
        self._BIT_RATE = None
        self._ROTATE = None
        self._RECORD_PATH = None
        self._RECORD_NAME = None
        self._SP = None
        self._STR_OUT = None
        self._STR_ERR = None

    def run(self):
        cmd = "adb shell screenrecord"
        if self._SIZE is not None:
            cmd += " --size %d" % self._SIZE
        if self._BIT_RATE is not None:
            cmd += " --bit-rate %d" % self._BIT_RATE
        if self._ROTATE is not None:
            cmd += " --rotate %d" % self._ROTATE
        if self._RECORD_PATH is None:
            self._RECORD_PATH = "/sdcard/"
        if self._RECORD_NAME is None:
            self._RECORD_NAME = time.strftime("%Y%m%d%H%M%S", time.localtime()) + ".mp4"
        cmd += ("  %s%s" % (self._RECORD_PATH, self._RECORD_NAME))
        self._SP = subprocess.Popen(self._DEVICE.make_cmd(cmd), stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        self._STR_OUT, self._STR_ERR = self._SP.communicate()
        time.sleep(0.5)

    def start_screen_record(self, size=None, bit_rate=None, rotate=None, record_path=None, record_name=None):
        self._SIZE = size
        self._BIT_RATE = bit_rate
        self._ROTATE = rotate
        self._RECORD_PATH = record_path
        self._RECORD_NAME = record_name
        self.start()

    def stop_screen_record(self, path=None):
        if self._SP is not None:
            self._SP.terminate()
            time.sleep(0.5)
            if path is not None:
                self.get_screen_record_file(path)
            self._SP = None

    def get_screen_record_file(self, path=None):
        if self._RECORD_PATH is not None and self._RECORD_NAME is not None:
            if path is None:
                self._DEVICE.adb_command(
                    "adb pull %s%s ./%s" % (self._RECORD_PATH, self._RECORD_NAME, self._RECORD_NAME))
            else:
                self._DEVICE.adb_command(
                    "adb pull %s%s %s%s" % (self._RECORD_PATH, self._RECORD_NAME, path, self._RECORD_NAME))
        self._RECORD_PATH = None
        self._RECORD_NAME = None



# coding=utf-8

"""
 :Decmdion:    命令类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2019年04月
"""

import subprocess
import threading
import queue
import os
import time


class Command(threading.Thread):
    def __init__(self, cmd=None):
        threading.Thread.__init__(self)
        self.result_callback = None
        self.result_queue = queue.Queue()
        if cmd is None:
            self.sp = None
        else:
            self.sp = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True, close_fds=True)
        self.exitFlag = True
        self.start()

    def run(self):
        """
        发送结果获取，线程主方法
        :return:
        """
        while self.exitFlag:
            if self.sp is not None:
                try:
                    line = self.sp.stdout.readline().decode("utf-8")
                    if not line:
                        break
                    if not self.result_queue.full():
                        self.result_queue.put(line)
                    if self.result_callback is not None:
                        self.result_callback(line)
                except Exception as e:
                    break
            # print("1:"+str(line))
            # print("2:"+str(self.result_queue.full()))
            # print("3:"+str(self.result_callback))
            # print("4:"+str(self.sp.pid))
        self.close()
        self.exitFlag = False
        self.sp.wait()

    def close(self):
        self.sp.send_signal(subprocess.signal.SIGTERM)
        self.sp.send_signal(subprocess.signal.CTRL_C_EVENT)
        self.sp.send_signal(subprocess.signal.CTRL_BREAK_EVENT)
        while self.sp.poll() is None:
            try:
                self.sp.terminate()
            except Exception as e:
                print(e.args)
        if self.sp.stdin:
            self.sp.stdin.close()
        if self.sp.stdout:
            self.sp.stdout.close()
        if self.sp.stderr:
            self.sp.stderr.close()
        try:
            self.sp.terminate()
            self.sp.kill()
        except OSError:
            pass

    def stop(self):
        """
        停止命令交互
        :return:
        """
        self.exitFlag = False
        self.close()
        self.sp.communicate()

    def read(self):
        """
        读取结果
        :return:结果
        """
        result = ""
        while not self.result_queue.empty() or self.exitFlag:
            # print("5:"+str(self.result_queue.empty()))
            # print("6:"+str(self.exitFlag))
            if not self.result_queue.empty():
                line = self.result_queue.get()
                if line != "\r\n":
                    result += line
        return result

    def read_line(self):
        """
        读取结果
        :return:结果
        """
        result = []
        while not self.result_queue.empty() or self.exitFlag:
            # print("5:"+str(self.result_queue.empty()))
            # print("6:"+str(self.exitFlag))
            if not self.result_queue.empty():
                line = self.result_queue.get()
                if line != "\r\n":
                    result.append(line.replace("\r\n", ""))
        return result

    def close_read(self):
        """
        结束命令线程并读取结果
        :return:读取的命令结果
        """
        self.close()
        return self.read()

    def close_read_line(self):
        """
        结束命令线程并读取结果
        :return:读取的命令结果
        """
        self.close()
        return self.read_line()

    def send(self, cmd):
        """
        发送系统命令
        :param cmd:  命令
        :return: 无
        """
        if self.sp is None:
            self.sp = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True, close_fds=True)
        else:
            cmd = (cmd + "\r\n").encode("utf-8")
            self.sp.stdin.write(cmd)
            self.sp.stdin.flush()

    def send_read(self, cmd):
        """
        发送系统命令并获取结果
        :param cmd:  命令
        :return: 命令结果
        """
        if self.sp is None:
            self.sp = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True, close_fds=True)
        else:
            cmd = (cmd + "\r\n").encode("utf-8")
            self.send(cmd)
        return self.read()

    def set_callback(self, func):
        """
        回调函数设置
        :param func: 回调函数
        :return: 无
        """
        self.result_callback = func



# encoding=utf-8
# Copyright 2013 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""
 :Description:    SurfaceStatsCollector(三方库引入)
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""
# import Queue
import datetime
import re
import threading

# Log marker containing SurfaceTexture timestamps.
_SURFACE_TEXTURE_TIMESTAMPS_MESSAGE = 'SurfaceTexture update timestamps'
_SURFACE_TEXTURE_TIMESTAMP_RE = '\d+'

_MIN_NORMALIZED_FRAME_LENGTH = 0.5


class SurfaceStatsCollector(object):
    """Collects surface stats for a SurfaceView from the output of SurfaceFlinger.

    Args:
      _device: the _device connection to use.
    """

    class Result(object):
        def __init__(self, name, value, unit):
            self.name = name
            self.value = value
            self.unit = unit

    def __init__(self, device):
        self._device = device
        self._collector_thread = None
        self._use_legacy_method = False
        self._surface_before = None
        self._get_data_event = None
        self._data_queue = None
        self._stop_event = None
        self._results = []
        self._warn_about_empty_data = True
        self._focuse_name = None

    def DisableWarningAboutEmptyData(self):
        self._focuse_name = self._focuse_name or self._device.get_focused_package_activity()
        self._warn_about_empty_data = False

    def Start(self):
        assert not self._collector_thread

        if self._ClearSurfaceFlingerLatencyData():
            self._get_data_event = threading.Event()
            self._stop_event = threading.Event()
            self._data_queue = Queue.Queue()
            self._collector_thread = threading.Thread(target=self._CollectorThread)
            self._collector_thread.start()
        else:
            self._use_legacy_method = True
            self._surface_before = self._GetSurfaceStatsLegacy()

    def Stop(self):
        self._StorePerfResults()
        if self._collector_thread:
            self._stop_event.set()
            self._collector_thread.join()
            self._collector_thread = None

    def SampleResults(self):
        self._StorePerfResults()
        results = self.GetResults()
        self._results = []
        return results

    def GetResults(self):
        return self._results  # or self._GetEmptyResults()

    def _GetEmptyResults(self):
        return [
            SurfaceStatsCollector.Result('refresh_period', None, 'seconds'),
            SurfaceStatsCollector.Result('jank_count', None, 'janks'),
            SurfaceStatsCollector.Result('max_frame_delay', None, 'vsyncs'),
            SurfaceStatsCollector.Result('frame_lengths', None, 'vsyncs'),
            SurfaceStatsCollector.Result('avg_surface_fps', None, 'fps')
        ]

    @staticmethod
    def _GetNormalizedDeltas(data, refresh_period, min_normalized_delta=None):
        deltas = [t2 - t1 for t1, t2 in zip(data, data[1:])]
        if min_normalized_delta != None:
            deltas = filter(lambda d: d / refresh_period >= min_normalized_delta,
                            deltas)
        return (deltas, [delta / refresh_period for delta in deltas])

    @staticmethod
    def _CalculateResults(refresh_period, timestamps, result_suffix):
        """Returns a list of SurfaceStatsCollector.Result."""
        frame_count = len(timestamps)
        seconds = timestamps[-1] - timestamps[0]

        frame_lengths, normalized_frame_lengths = \
            SurfaceStatsCollector._GetNormalizedDeltas(
                timestamps, refresh_period, _MIN_NORMALIZED_FRAME_LENGTH)
        # if len(frame_lengths) < frame_count - 1:
        #     print('Skipping frame lengths that are too short.')
        #     frame_count = len(frame_lengths) + 1
        # if len(frame_lengths) == 0:
        #     raise Exception('No valid frames lengths found.')
        length_changes, normalized_changes = \
            SurfaceStatsCollector._GetNormalizedDeltas(
                frame_lengths, refresh_period)
        jankiness = [max(0, round(change)) for change in normalized_changes]
        pause_threshold = 20
        jank_count = sum(1 for change in jankiness
                         if change > 0 and change < pause_threshold)
        return [
            SurfaceStatsCollector.Result(
                'avg_surface_fps' + result_suffix,
                int(round((frame_count - 1) / seconds)), 'fps'),
            SurfaceStatsCollector.Result(
                'jank_count' + result_suffix, jank_count, 'janks'),
            SurfaceStatsCollector.Result(
                'max_frame_delay' + result_suffix,
                round(max(normalized_frame_lengths)),
                'vsyncs'),
            SurfaceStatsCollector.Result(
                'frame_lengths' + result_suffix, normalized_frame_lengths,
                'vsyncs'),
        ]

    @staticmethod
    def _CalcPerFps(unit, timestamps):
        per_fps = []
        start_time = timestamps[0]
        tmp_index = 0
        for index, time_one in enumerate(timestamps):
            if time_one - start_time >= unit:
                per_fps.append(index - tmp_index)
                start_time = time_one
                tmp_index = index
        return per_fps

    @staticmethod
    def _CalculateBuckets(refresh_period, timestamps):
        results = []
        for pct in [0.99, 0.5]:
            sliced = timestamps[min(int(-pct * len(timestamps)), -3):]
            results += SurfaceStatsCollector._CalculateResults(
                refresh_period, sliced, '_' + str(int(pct * 100)))
        return results

    def _CalcFps(self, timestamps):
        if len(timestamps) > 0:
            fps = []
            time_a = timestamps[0]
            for time in timestamps[1:]:
                fps.append(int((time - time_a) * 1000))
                time_a = time
            return fps
        else:
            return None

    @staticmethod
    def _CalculateJankPercent(refresh_period, timestamps_a, timestamps_c):
        deltas = [t2 - t1 for t1, t2 in zip(timestamps_a, timestamps_c)]
        total = len(deltas)
        jank_percent_60 = sum(1 for delta in deltas if delta / refresh_period <= 1)
        jank_percent_30 = sum(1 for delta in deltas if delta / refresh_period <= 2)
        jank_percent_15 = sum(1 for delta in deltas if delta / refresh_period <= 4)
        return [
            SurfaceStatsCollector.Result('jank_percent_60', round(jank_percent_60 * 100 / round(total), 2), '%'),
            SurfaceStatsCollector.Result('jank_percent_30', round(jank_percent_30 * 100 / round(total), 2), '%'),
            SurfaceStatsCollector.Result('jank_percent_15', round(jank_percent_15 * 100 / round(total), 2), '%')
        ]

    def _StorePerfResults(self):
        if self._use_legacy_method:
            surface_after = self._GetSurfaceStatsLegacy()
            td = surface_after['timestamp'] - self._surface_before['timestamp']
            seconds = td.seconds + td.microseconds / 1e6
            frame_count = (surface_after['page_flip_count'] -
                           self._surface_before['page_flip_count'])
            self._results.append(SurfaceStatsCollector.Result(
                'avg_surface_fps', int(round(frame_count / seconds)), 'fps'))
            return

        # Non-legacy method.
        assert self._collector_thread
        (refresh_period, timestamps, timestamps_a, timestamps_c) = self._GetDataFromThread()
        if not refresh_period or not len(timestamps) >= 3:
            if self._warn_about_empty_data:
                print('Surface stat data is empty')
            return
        self._results.append(SurfaceStatsCollector.Result('refresh_period', refresh_period, 'seconds'))
        self._results.append(SurfaceStatsCollector.Result('per_fps', self._CalcPerFps(1, timestamps), 'per_fps'))
        self._results.append(SurfaceStatsCollector.Result('fps', self._CalcFps(timestamps), 'ms'))
        self._results += self._CalculateResults(refresh_period, timestamps, '')
        self._results += self._CalculateBuckets(refresh_period, timestamps)
        self._results += self._CalculateJankPercent(refresh_period, timestamps_a, timestamps_c)

    def _CollectorThread(self):
        last_timestamp = 0
        last_timestamp_a = 0
        last_timestamp_c = 0
        timestamps = []
        timestamps_a = []
        timestamps_c = []
        retries = 0

        while not self._stop_event.is_set():
            self._get_data_event.wait(1)
            try:
                refresh_period, new_timestamps, new_timestamps_a, new_timestamps_c = self._GetSurfaceFlingerFrameData()
                if refresh_period is None or timestamps is None:
                    retries += 1
                    if retries < 3:
                        continue
                    if last_timestamp:
                        # Some data has already been collected, but either the app
                        # was closed or there's no new data. Signal the main thread and
                        # wait.
                        self._data_queue.put((None, None))
                        self._stop_event.wait()
                        break
                    raise Exception('Unable to get surface flinger latency data')

                timestamps += [timestamp for timestamp in new_timestamps if timestamp > last_timestamp]
                timestamps_a += [timestamp_a for timestamp_a in new_timestamps_a if timestamp_a > last_timestamp_a]
                timestamps_c += [timestamp_c for timestamp_c in new_timestamps_c if timestamp_c > last_timestamp_c]
                if len(timestamps):
                    last_timestamp = timestamps[-1]
                if len(timestamps_a):
                    last_timestamp_a = timestamps_a[-1]
                if len(timestamps_c):
                    last_timestamp_c = timestamps_c[-1]
                if self._get_data_event.is_set():
                    self._get_data_event.clear()
                    self._data_queue.put((refresh_period, timestamps, timestamps_a, timestamps_c))
                    timestamps = []
                    timestamps_a = []
                    timestamps_c = []
            except Exception as e:
                # On any error, before aborting, put the exception into _data_queue to
                # prevent the main thread from waiting at _data_queue.get() infinitely.
                self._data_queue.put(e)
                raise

    def _GetDataFromThread(self):
        self._get_data_event.set()
        ret = self._data_queue.get()
        if isinstance(ret, Exception):
            raise ret
        return ret

    def _ClearSurfaceFlingerLatencyData(self):
        """Clears the SurfaceFlinger latency data.

        Returns:
          True if SurfaceFlinger latency is supported by the device, otherwise
          False.
        """
        # The command returns nothing if it is supported, otherwise returns many
        # lines of result just like 'dumpsys SurfaceFlinger'.
        results = self._device.script(
            'adb shell dumpsys SurfaceFlinger --latency-clear ' + self._focuse_name)
        return not len(results)

    def _GetSurfaceFlingerFrameData(self):
        """Returns collected SurfaceFlinger frame timing data.

        Returns:
          A tuple containing:
          - The display's nominal refresh period in seconds.
          - A list of timestamps signifying frame presentation times in seconds.
          The return value may be (None, None) if there was no data collected (for
          example, if the app was closed before the collector thread has finished).
        """
        # adb shell dumpsys SurfaceFlinger --latency <window name>
        # prints some information about the last 128 frames displayed in
        # that window.
        # The data returned looks like this:
        # 16954612
        # 7657467895508   7657482691352   7657493499756
        # 7657484466553   7657499645964   7657511077881
        # 7657500793457   7657516600576   7657527404785
        # (...)
        #
        # The first line is the refresh period (here 16.95 ms), it is followed
        # by 128 lines w/ 3 timestamps in nanosecond each:
        # A) when the app started to draw
        # B) the vsync immediately preceding SF submitting the frame to the h/w
        # C) timestamp immediately after SF submitted that frame to the h/w
        #
        # The difference between the 1st and 3rd timestamp is the frame-latency.
        # An interesting data is when the frame latency crosses a refresh period
        # boundary, this can be calculated this way:
        #
        # ceil((C - A) / refresh-period)
        #
        # (each time the number above changes, we have a "jank").
        # If this happens a lot during an animation, the animation appears
        # janky, even if it runs at 60 fps in average.
        #
        # We use the special "SurfaceView" window name because the statistics for
        # the activity's main window are not updated when the main web content is
        # composited into a SurfaceView.
        # print (self._focuse_name)
        results = self._device.script('adb shell dumpsys SurfaceFlinger --latency ' + self._focuse_name).split("\r\n")
        if not len(results):
            return (None, None)
        timestamps = []
        timestamps_a = []
        timestamps_c = []
        nanoseconds_per_second = 1e9
        refresh_period = long(results[0]) / nanoseconds_per_second

        # If a fence associated with a frame is still pending when we query the
        # latency data, SurfaceFlinger gives the frame a timestamp of INT64_MAX.
        # Since we only care about completed frames, we will ignore any timestamps
        # with this value.
        pending_fence_timestamp = (1 << 63) - 1

        for line in results[1:]:
            fields = line.split()
            if len(fields) != 3:
                continue
            timestamp = long(fields[1])
            timestamp_a = long(fields[0])
            timestamp_c = long(fields[2])
            if timestamp == pending_fence_timestamp:
                continue
            timestamp /= nanoseconds_per_second
            timestamp_a /= nanoseconds_per_second
            timestamp_c /= nanoseconds_per_second
            timestamps.append(timestamp)
            timestamps_a.append(timestamp_a)
            timestamps_c.append(timestamp_c)
        # print (refresh_period)
        # print (timestamps)
        return (refresh_period, timestamps, timestamps_a, timestamps_c)

    def _GetSurfaceStatsLegacy(self):
        """Legacy method (before JellyBean), returns the current Surface index
           and timestamp.

        Calculate FPS by measuring the difference of Surface index returned by
        SurfaceFlinger in a period of time.

        Returns:
          Dict of {page_flip_count (or 0 if there was an error), timestamp}.
        """
        results = self._device.script('adb shell service call SurfaceFlinger 1013')
        if "Operation not permitted" in results:
            results = self.adb_interface.script("adb shell su -k service call SurfaceFlinger 1013").split("\r\n")
        else:
            results = results.split("\r\n")
        assert len(results) == 1
        match = re.search('^Result: Parcel\((\w+)', results[0])
        cur_surface = 0
        if match:
            try:
                cur_surface = int(match.group(1), 16)
            except Exception:
                print('Failed to parse current surface from ' + match.group(1))
        else:
            print('Failed to call SurfaceFlinger surface ' + results[0])
        return {
            'page_flip_count': cur_surface,
            'timestamp': datetime.datetime.now(),
        }



# coding=utf-8

"""
 :Deadb_commandion:    设备类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""
import os
import re
import platform

PATH = lambda p: os.path.abspath(os.path.join(os.path.dirname(__file__), p))

# 判断系统类型，windows使用findstr，linux使用grep
sys_info = platform.system()
if sys_info is "Windows":
    find_util = "findstr"
else:
    find_util = "grep"


class Memory(object):
    def __init__(self, device=None, package=None):
        self._meminfo_text_ = None
        self._native_heap_pss_total_ = None
        self._native_heap_private_dirty_ = None
        self._native_heap_private_clean_ = None
        self._native_heap_swapped_dirty_ = None
        self._native_heap_heap_size_ = None
        self._native_heap_heap_alloc_ = None
        self._native_heap_heap_free_ = None
        self._dalvik_heap_pss_total_ = None
        self._dalvik_heap_private_dirty_ = None
        self._dalvik_heap_private_clean_ = None
        self._dalvik_heap_swapped_dirty_ = None
        self._dalvik_heap_heap_size_ = None
        self._dalvik_heap_heap_alloc_ = None
        self._dalvik_heap_heap_free_ = None
        self._dalvik_alloc_pss_total_ = None
        self._dalvik_alloc_private_dirty_ = None
        self._dalvik_alloc_private_clean_ = None
        self._dalvik_alloc_swapped_dirty_ = None
        self._stack_pss_total_ = None
        self._stack_private_dirty_ = None
        self._stack_private_clean_ = None
        self._stack_swapped_dirty_ = None
        self._cursor_pss_total_ = None
        self._cursor_private_dirty_ = None
        self._cursor_private_clean_ = None
        self._cursor_swapped_dirty_ = None
        self._ashmem_pss_total_ = None
        self._ashmem_private_dirty_ = None
        self._ashmem_private_clean_ = None
        self._ashmem_swapped_dirty_ = None
        self._other_dev_pss_total_ = None
        self._other_dev_private_dirty_ = None
        self._other_dev_private_clean_ = None
        self._other_dev_swapped_dirty_ = None
        self._gfx_dev_pss_total_ = None
        self._gfx_dev_private_dirty_ = None
        self._gfx_dev_private_clean_ = None
        self._gfx_dev_swapped_dirty_ = None
        self._alloc_dev_pss_total_ = None
        self._alloc_dev_private_dirty_ = None
        self._alloc_dev_private_clean_ = None
        self._alloc_dev_swapped_dirty_ = None
        self._file_so_mmap_pss_total_ = None
        self._file_so_mmap_private_dirty_ = None
        self._file_so_mmap_private_clean_ = None
        self._file_so_mmap_swapped_dirty_ = None
        self._file_apk_mmap_pss_total_ = None
        self._file_apk_mmap_private_dirty_ = None
        self._file_apk_mmap_private_clean_ = None
        self._file_apk_mmap_swapped_dirty_ = None
        self._file_ttf_mmap_pss_total_ = None
        self._file_ttf_mmap_private_dirty_ = None
        self._file_ttf_mmap_private_clean_ = None
        self._file_ttf_mmap_swapped_dirty_ = None
        self._file_dex_mmap_pss_total_ = None
        self._file_dex_mmap_private_dirty_ = None
        self._file_dex_mmap_private_clean_ = None
        self._file_dex_mmap_swapped_dirty_ = None
        self._file_oat_mmap_pss_total_ = None
        self._file_oat_mmap_private_dirty_ = None
        self._file_oat_mmap_private_clean_ = None
        self._file_oat_mmap_swapped_dirty_ = None
        self._file_art_mmap_pss_total_ = None
        self._file_art_mmap_private_dirty_ = None
        self._file_art_mmap_private_clean_ = None
        self._file_art_mmap_swapped_dirty_ = None
        self._other_mmap_pss_total_ = None
        self._other_mmap_private_dirty_ = None
        self._other_mmap_private_clean_ = None
        self._other_mmap_swapped_dirty_ = None
        self._alloc_mmap_pss_total_ = None
        self._alloc_mmap_private_dirty_ = None
        self._alloc_mmap_private_clean_ = None
        self._alloc_mmap_swapped_dirty_ = None
        self._gl_mtrack_pss_total_ = None
        self._gl_mtrack_private_dirty_ = None
        self._gl_mtrack_private_clean_ = None
        self._gl_mtrack_swapped_dirty_ = None
        self._egl_mtrack_pss_total_ = None
        self._egl_mtrack_private_dirty_ = None
        self._egl_mtrack_private_clean_ = None
        self._egl_mtrack_swapped_dirty_ = None
        self._unknown_pss_total_ = None
        self._unknown_private_dirty_ = None
        self._unknown_private_clean_ = None
        self._unknown_swapped_dirty_ = None
        self._total_pss_total_ = None
        self._total_private_dirty_ = None
        self._total_private_clean_ = None
        self._total_swapped_dirty_ = None
        self._total_heap_size_ = None
        self._total_heap_alloc_ = None
        self._total_heap_free_ = None
        self._device_ = None
        self._package_ = package or self._device_.get_focused_package_activity().split("\\")[0]
        """
        对象初始化
        """
        self._device_ = device

    def get_meminfo_text(self):
        """
        获取内存信息
        """
        self._meminfo_text_ = self._device_.adb_command("adb shell dumpsys meminfo %s" % self._package_)
        pass

    def update(self):
        self.get_meminfo_text()
        self.native_heap()
        self.dalvik_heap()
        self.dalvik_alloc()
        self.stack()
        self.cursor()
        self.ashmem()
        self.gfx_dev()
        self.alloc_dev()
        self.file_so_mmap()
        self.file_apk_mmap()
        self.file_ttf_mmap()
        self.file_dex_mmap()
        self.file_oat_mmap()
        self.file_art_mmap()
        self.alloc_mmap()
        self.unknown()
        self.total()
        self.other_dev()
        self.other_mmap()
        self.egl_mtrack()
        self.gl_mtrack()

    def get_memory_list(self, category):
        """
        获取某项的内存信息
        """
        if self._meminfo_text_ is not None:
            meminfo_list = self._meminfo_text_.split("\n")
            for meminfo in meminfo_list:
                if category in meminfo:
                    pattern = re.compile(r"\d+")
                    ret = pattern.findall(meminfo)
                    return ret
            return None
        else:
            return None

    def native_heap(self):
        meminfo = self.get_memory_list("Native Heap")
        if meminfo is not None:
            self._native_heap_pss_total_ = meminfo[0]
            self._native_heap_private_dirty_ = meminfo[1]
            self._native_heap_private_clean_ = meminfo[2]
            self._native_heap_swapped_dirty_ = meminfo[3]
            self._native_heap_heap_size_ = meminfo[4]
            self._native_heap_heap_alloc_ = meminfo[5]
            self._native_heap_heap_free_ = meminfo[6]
        return meminfo

    def dalvik_heap(self):
        meminfo = self.get_memory_list("Dalvik Heap")
        if meminfo is not None:
            self._dalvik_heap_pss_total_ = meminfo[0]
            self._dalvik_heap_private_dirty_ = meminfo[1]
            self._dalvik_heap_private_clean_ = meminfo[2]
            self._dalvik_heap_swapped_dirty_ = meminfo[3]
            self._dalvik_heap_heap_size_ = meminfo[4]
            self._dalvik_heap_heap_alloc_ = meminfo[5]
            self._dalvik_heap_heap_free_ = meminfo[6]
        return meminfo

    def dalvik_alloc(self):
        meminfo = self.get_memory_list("Dalvik Alloc")
        if meminfo is not None:
            self._dalvik_alloc_pss_total_ = meminfo[0]
            self._dalvik_alloc_private_dirty_ = meminfo[1]
            self._dalvik_alloc_private_clean_ = meminfo[2]
            self._dalvik_alloc_swapped_dirty_ = meminfo[3]
        return meminfo

    def stack(self):
        meminfo = self.get_memory_list("Stack")
        if meminfo is not None:
            self._stack_pss_total_ = meminfo[0]
            self._stack_private_dirty_ = meminfo[1]
            self._stack_private_clean_ = meminfo[2]
            self._stack_swapped_dirty_ = meminfo[3]
        return meminfo

    def cursor(self):
        meminfo = self.get_memory_list("Cursor")
        if meminfo is not None:
            self._cursor_pss_total_ = meminfo[0]
            self._cursor_private_dirty_ = meminfo[1]
            self._cursor_private_clean_ = meminfo[2]
            self._cursor_swapped_dirty_ = meminfo[3]
        return meminfo

    def ashmem(self):
        meminfo = self.get_memory_list("Ashmem")
        if meminfo is not None:
            self._ashmem_pss_total_ = meminfo[0]
            self._ashmem_private_dirty_ = meminfo[1]
            self._ashmem_private_clean_ = meminfo[2]
            self._ashmem_swapped_dirty_ = meminfo[3]
        return meminfo

    def gfx_dev(self):
        meminfo = self.get_memory_list("Gfx Dev")
        if meminfo is not None:
            self._gfx_dev_pss_total_ = meminfo[0]
            self._gfx_dev_private_dirty_ = meminfo[1]
            self._gfx_dev_private_clean_ = meminfo[2]
            self._gfx_dev_swapped_dirty_ = meminfo[3]
        return meminfo

    def alloc_dev(self):
        meminfo = self.get_memory_list("Alloc Dev")
        if meminfo is not None:
            self._alloc_dev_pss_total_ = meminfo[0]
            self._alloc_dev_private_dirty_ = meminfo[1]
            self._alloc_dev_private_clean_ = meminfo[2]
            self._alloc_dev_swapped_dirty_ = meminfo[3]
        return meminfo

    def other_dev(self):
        meminfo = self.get_memory_list("Other dev")
        if meminfo is not None:
            self._other_dev_pss_total_ = meminfo[0]
            self._other_dev_private_dirty_ = meminfo[1]
            self._other_dev_private_clean_ = meminfo[2]
            self._other_dev_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_so_mmap(self):
        meminfo = self.get_memory_list(".so mmap")
        if meminfo is not None:
            self._file_so_mmap_pss_total_ = meminfo[0]
            self._file_so_mmap_private_dirty_ = meminfo[1]
            self._file_so_mmap_private_clean_ = meminfo[2]
            self._file_so_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_apk_mmap(self):
        meminfo = self.get_memory_list(".apk mmap")
        if meminfo is not None:
            self._file_apk_mmap_pss_total_ = meminfo[0]
            self._file_apk_mmap_private_dirty_ = meminfo[1]
            self._file_apk_mmap_private_clean_ = meminfo[2]
            self._file_apk_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_ttf_mmap(self):
        meminfo = self.get_memory_list(".ttf mmap")
        if meminfo is not None:
            self._file_ttf_mmap_pss_total_ = meminfo[0]
            self._file_ttf_mmap_private_dirty_ = meminfo[1]
            self._file_ttf_mmap_private_clean_ = meminfo[2]
            self._file_ttf_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_dex_mmap(self):
        meminfo = self.get_memory_list(".dex mmap")
        if meminfo is not None:
            self._file_dex_mmap_pss_total_ = meminfo[0]
            self._file_dex_mmap_private_dirty_ = meminfo[1]
            self._file_dex_mmap_private_clean_ = meminfo[2]
            self._file_dex_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_oat_mmap(self):
        meminfo = self.get_memory_list(".oat mmap")
        if meminfo is not None:
            self._file_oat_mmap_pss_total_ = meminfo[0]
            self._file_oat_mmap_private_dirty_ = meminfo[1]
            self._file_oat_mmap_private_clean_ = meminfo[2]
            self._file_oat_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def file_art_mmap(self):
        meminfo = self.get_memory_list(".art mmap")
        if meminfo is not None:
            self._file_art_mmap_pss_total_ = meminfo[0]
            self._file_art_mmap_private_dirty_ = meminfo[1]
            self._file_art_mmap_private_clean_ = meminfo[2]
            self._file_art_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def other_mmap(self):
        meminfo = self.get_memory_list("Other mmap")
        if meminfo is not None:
            self._other_mmap_pss_total_ = meminfo[1]
            self._other_mmap_private_dirty_ = meminfo[1]
            self._other_mmap_private_clean_ = meminfo[2]
            self._other_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def alloc_mmap(self):
        meminfo = self.get_memory_list("Alloc mmap")
        if meminfo is not None:
            self._alloc_mmap_pss_total_ = meminfo[0]
            self._alloc_mmap_private_dirty_ = meminfo[1]
            self._alloc_mmap_private_clean_ = meminfo[2]
            self._alloc_mmap_swapped_dirty_ = meminfo[3]
        return meminfo

    def egl_mtrack(self):
        meminfo = self.get_memory_list("EGL mtrack")
        if meminfo is not None:
            self._egl_mtrack_pss_total_ = meminfo[0]
            self._egl_mtrack_private_dirty_ = meminfo[1]
            self._egl_mtrack_private_clean_ = meminfo[2]
            self._egl_mtrack_swapped_dirty_ = meminfo[3]
        return meminfo

    def gl_mtrack(self):
        meminfo = self.get_memory_list("GL mtrack")
        if meminfo is not None:
            self._gl_mtrack_pss_total_ = meminfo[0]
            self._gl_mtrack_private_dirty_ = meminfo[1]
            self._gl_mtrack_private_clean_ = meminfo[2]
            self._gl_mtrack_swapped_dirty_ = meminfo[3]
        return meminfo

    def unknown(self):
        meminfo = self.get_memory_list("Unknown")
        if meminfo is not None:
            self._unknown_pss_total_ = meminfo[0]
            self._unknown_private_dirty_ = meminfo[1]
            self._unknown_private_clean_ = meminfo[2]
            self._unknown_swapped_dirty_ = meminfo[3]
        return meminfo

    def total(self):
        meminfo = self.get_memory_list("TOTAL")
        if meminfo is not None:
            print(meminfo)
            self._total_pss_total_ = meminfo[0]
            self._total_private_dirty_ = meminfo[1]
            self._total_private_clean_ = meminfo[2]
            self._total_swapped_dirty_ = meminfo[3]
            self._total_heap_size_ = meminfo[4]
            self._total_heap_alloc_ = meminfo[5]
            self._total_heap_free_ = meminfo[6]
        return meminfo



# coding=utf-8
import re
from ATC.Device import Device
from ATC.Command import Command

__version__ = '1.0'
__author__ = 'bony'
# __all__ = ['BAndroidDriver', 'Element']
"""
ATD
"""


def get_device(serial=None, name=None):
    devices = get_devices()
    if len(devices) > 0:
        if serial is None:
            if name is None:
                return devices[0]
            else:
                device = devices[0]
                device._name = name
                return device
        else:
            if get_by_serial_device(serial) is None:
                print(u"未找到匹配设备!")
                return None
            else:
                device = get_by_serial_device(serial)
                if name is None:
                    return device
                else:
                    device._name = name
                    return device
    else:
        print(u"未连接设备!")
        return None


def existence_device(device):
    if device.get_serial() is None:
        return False
    else:
        return True


def get_by_serial_device(serial):
    devices = get_devices()
    for device in devices:
        if device.get_serial() == serial:
            return device
    return None


def get_devices():
    devices = []
    for line in Command("adb devices -l").read_line():
        if "device" in line and "product" in line:
            devices_list = re.compile(r"((?:[a-zA-Z0-9\w]+))").findall(line)
            devices.append(Device(devices_list[0], devices_list[3]))
    return devices



# coding=utf-8
import re
import platform

'''
 :Deadb_commandion:    系统操作类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
'''

# from Core.Utils.adb_interface import AdbInterface

# adb_interface = AdbInterface()

# 判断系统类型，windows使用findstr，linux使用grep
system_info = platform.system()
if system_info is "Windows":
    find_util = "findstr"
else:
    find_util = "grep"


class SystemInfo(object):
    device = None

    def __init__(self, device=None):
        self.device = device

    def getCpukinds(self):
        return self.device.adb_command("adb shell getprop ro.product.cpu.abi")

    def getDeviceState(self):
        """
        获取设备状态： offline | bootloader | device
         usage: getDeviceState()
        """
        return self.device.adb_command("adb get-state").split("\n")[0]

    def getDeviceID(self):
        """
        获取设备id号，return serialNo
         usage: getDeviceID()
        """
        return self.device.adb_command("adb get-serialno").split("\n")[0]

    def getDeviceIDlist(self):
        """
        获取设备id号，return list(serialNo)
         usage: getDeviceIDlist()
        """
        l = []
        tmp = []
        l = self.device.adb_command("adb devices").split("\r\n")
        del l[0]
        for x in l:
            if x != "":
                tmp.append(x.split("\t")[0].strip('\''))
        return tmp

    def getAppList(self):
        """
        获取设备中安装的应用包名列表
         usage: getAppList()
        """
        app = []
        for packages in self.device.adb_command("adb shell pm list packages").split():
            app.append(packages.split(":")[1])

        return app

    def getSystemAppList(self):
        """
        获取设备中安装的系统应用包名列表
         usage: getSystemAppList()
        """
        sysApp = []
        for packages in self.device.adb_command("adb shell pm list packages -s").split():
            sysApp.append(packages.split(":")[1])

        return sysApp

    def getThirdAppList(self):
        """
        获取设备中安装的第三方应用包名列表
         usage: getThirdAppList()
        """
        thirdApp = []
        for packages in self.device.adb_command("adb shell pm list packages -3").split():
            thirdApp.append(packages.split(":")[1])

        return thirdApp

    def getMatchingAppList(self, keyword):
        """
        模糊查询与keyword匹配的应用包名列表
         args:
        - keyword -: 关键字
        usage: getMatchingAppList("qq")
        """
        matApp = []
        for packages in self.device.adb_command("adb shell pm list packages " + keyword).split():
            matApp.append(packages.split(":")[1])
        return matApp

    def getAppAddressFromPname(self, packagename):
        """
        根据包名查询应用地址
        args:
        - packagename -: 包名
        usage: getAppAddressFromPname("com.android.test")
        """
        address = self.device.adb_command("adb shell pm list packages -f |" + find_util + " " + packagename)
        return address.split(":")[1].split("=")[0]

    def getAppAddressFromKeyList(self, key):
        """
        根据关键字查找应用地址
        args:
        - key -: 关键字
        usage: getAppAddressFromKeyList("android")
        """
        l = []
        tmp = []
        l = self.device.adb_command("adb shell pm list packages -f |" + find_util + " " + key).split("\n")
        for x in l:
            if x != "":
                tmp.append(x.split(":")[1].split("=")[0])
        return tmp

    def getAppAddressList(self):
        """
        获取安装应用地址
        usage: getAppAddressList()
        """
        l = []
        tmp = []
        l = self.device.adb_command("adb shell pm list packages -f").split("\n")
        for x in l:
            if x != "":
                tmp.append(x.split(":")[1].split("=")[0])
        return tmp

    def getAppNo(self):
        """
        获取应用数量
        usage: getAppNo()
        """
        return len(self.getAppList())

    def get_sys_appno(self):
        """
        获取系统应用数量
        usage: get_sys_appNo()
        """
        return len(self.getSystemAppList())

    def get_third_appno(self):
        """
        获取第三方应用数量
        usage: getThirdAppNo()
        """
        return len(self.getThirdAppList())

    def get_sdk_version(self):
        """
        得到sdk版本号
        usage: getSdkVersion()
        """
        return self.device.adb_command("adb shell getprop ro.build.version.sdk").split("\r\n")[0]

    def get_cur_handle(self):
        """
        得到当前的handle
        usage: getCurHandle()
        """
        fname = []
        l = self.device.adb_command("adb shell dumpsys SurfaceFlinger").split("\r\n")
        n = 0
        z = 0
        for x in l:
            n += 1
            if "----------+-" in x:
                fname = l[n - 2].split("|")
                break
        for y in fname:
            z += 1
            if "handle" in y:
                handle = l[n].split("|")
                return handle[z - 1]

    def get_screen_resolution(self):
        """
        获取设备屏幕分辨率，return (width, high)
        usage: getScreenResolution()
        """
        if int(self.get_sdk_version()) < 14:  # 如果版本低于4。0，返回一个320,480
            return 320, 480
        pattern = re.compile(r"\d+")
        try:
            out = self.device.adb_command("adb shell dumpsys display | " + find_util + " PhysicalDisplayInfo")
            display = pattern.findall(out)
            if len(display) < 1:
                raise Exception('Unable to get PhysicalDisplayInfo' + str(out))
            else:
                return int(display[0]), int(display[1])
        except Exception as e:
            out = self.device.adb_command("adb shell wm size")
            display = pattern.findall(out)
            if len(display) < 1:
                raise Exception('Unable to get PhysicalDisplayInfo' + str(out))
            else:
                return int(display[0]), int(display[1])



# coding=utf-8

"""
 :Deadb_commandion:    设备类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""
import os
import re
# import time
# import subprocess
# import threading
# import inspect
# import ctypes
import platform
from ATC.Element import Element
from ATC.Memory import Memory
from ATC.System import SystemInfo
from ATC.ScreenRecord import ScreenRecord
from ATC.Command import Command
# from SurfaceStatsCollector import SurfaceStatsCollector
import xml.etree.cElementTree as ET

PATH = lambda p: os.path.abspath(os.path.join(os.path.dir_NAME(__file__), p))

# 判断系统类型，windows使用findstr，linux使用grep
sys_info = platform.system()
if sys_info is "Windows":
    find_util = "findstr"
else:
    find_util = "grep"


class Device(object):
    def __init__(self, _SERIAL=None, _NAME=None):
        self._DEBUG = False
        self._WIDTH = None
        self._HIGH = None
        self._SERIAL = _SERIAL
        self._NAME = _NAME
        self._SYSTEM = self.system()
        self.ScreenRecord = ScreenRecord(self)
        # self._SurfaceStatsCollector_ = SurfaceStatsCollector(self)

    def init_screen_size(self):
        try:
            wh = self._SYSTEM.get_screen_resolution()
            self._WIDTH = wh[0]
            self._HIGH = wh[1]
            return True
        except Exception as e:
            print(e.message)
            # raise e

    def adb_command(self, cmd):
        return Command(self.make_cmd(cmd)).read()

    def make_cmd(self, cmd):
        if self._SERIAL is not None:
            cmd = 'adb -s ' + self._SERIAL + ' ' + cmd.split('adb')[1]
        if self._DEBUG is True:
            print(cmd)
        return cmd

    def set_debug(self, boolean):
        """
        设置debug值
        """
        self._DEBUG = boolean

    def get_debug(self):
        """
        获取debug值
        """
        return self._DEBUG

    def get_name(self):
        """
        获取设备名称
        """
        return self._NAME

    def get_serial(self):
        """
        获取设备序列号
        """
        return self._SERIAL

    def rm(self, path):
        """
        删除文件
        """
        self.adb_command('adb shell rm -rf ' + path)

    def sdcard_rm(self, path):
        """
        删除sdcard中的文件
        """
        path = '/mnt/sdcard' + path
        self.rm(path)

    def get_layout_xml(self, path):
        self.adb_command('adb shell uiautomator dump --compressed /data/local/tmp/LayoutXml.xml')
        self.adb_command('adb pull /data/local/tmp/LayoutXml.xml ' + PATH(path))
        self.adb_command('adb shell rm -r /data/local/tmp/LayoutXml.xml')

    def get_screencap(self, path=None):
        """
        get screencap
        :param path: 存放文件夹路径
        :return: file <png>
        """
        self.adb_command('adb shell /system/bin/screencap -p /sdcard/screencap.png')
        if path is None:
            self.adb_command('adb pull /sdcard/screencap.png ./screencap.png')
        else:
            self.adb_command('adb pull /sdcard/screencap.png ' + path)
        self.adb_command("adb shell rm /sdcard/screencap.png")

    def click(self, x, y):
        """
        点击坐标
        :param x:横坐标
        :param y:纵坐标
        :return:None
        """
        self.adb_command('adb shell input tap %s %s' % (x, y))

    def click_back(self):
        """
        click Back
        :return:
        """
        self.click_keycode('KEYCODE_BACK')

    def click_star(self):
        """
        click star
        :return:
        """
        self.click_keycode('KEYCODE_STAR')

    def click_enter(self):
        """
        click Enter
        :return:
        """
        self.click_keycode('KEYCODE_ENTER')

    def swipe(self, x, y, x1, y1):
        """
        swipe
        :param x: 起始横坐标
        :param y: 起始纵坐标
        :param x1: 终止横坐标
        :param y1: 终止纵坐标
        :return:
        """
        self.adb_command('adb shell input swipe %s %s %s %s' % (x, y, x1, y1))

    def swipe_to_left(self, proportion):
        """
        向左滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
        self.swipe(
            self._WIDTH * (0.5 + 0.5 * proportion), self._HIGH * 0.5,
            self._WIDTH * (0.5 - 0.5 * proportion), self._HIGH * 0.5)

    def swipe_to_right(self, proportion):
        """
        向右滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
        self.swipe(
            self._WIDTH * (0.5 - 0.5 * proportion), self._HIGH * 0.5,
            self._WIDTH * (0.5 + 0.5 * proportion), self._HIGH * 0.5)

    def swipe_to_up(self, proportion):
        """
        向上滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
        self.swipe(
            self._WIDTH * 0.5, self._HIGH * (0.5 + 0.5 * proportion),
            self._WIDTH * 0.5, self._HIGH * (0.5 - 0.5 * proportion))

    def swipe_to_down(self, proportion):
        """
        向下滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
        self.swipe(
            self._WIDTH * 0.5, self._HIGH * (0.5 - 0.5 * proportion),
            self._WIDTH * 0.5, self._HIGH * (0.5 + 0.5 * proportion))

    def long_click(self, x, y):
        """
        长按某一位置
        :param x: 横坐标
        :param y: 纵坐标
        :return:
        """
        self.adb_command('adb shell input swipe %s %s %s %s 2000' % (x, y, x, y))

    def input(self, text):
        """
        input txt
        :param text:
        :return:
        """
        self.adb_command('adb shell input text ' + text)

    def click_keycode(self, keycode):
        """
        tap Key code
        :param keycode:
        :return:
        """
        self.adb_command('adb shell input keyevent ' + keycode)

    def long_click_keycode(self, keycode):
        """
        long Press Key code
        :param keycode:
        :return:
        """
        self.adb_command('input keyevent --longpress ' + keycode)

    def install_app(self, apk_path):
        """
        根据app路径安装app
        :param apk_path: app file path
        :return:
        """
        self.adb_command('adb install -r ' + apk_path)

    def stop_app(self, package):
        """
        stop app
        :param package:包名
        :return:
        """
        self.adb_command('adb shell am force-stop ' + package)

    def start_app(self, package, activity):
        """
        Start App
        :param package:包名
        :param activity:主视图
        :return:
        """
        self.adb_command('adb shell am start ' + package + '/' + activity)

    def app_clear_data(self, package):
        """
        Clear App Data
        :param package:包名
        :return:
        """
        self.adb_command('adb shell pm clear ' + package)

    def wifi_stop(self):
        """
        wifi stop
        """
        self.adb_command('adb shell svc wifi disable')

    def wifi_start(self):
        """
        wifi start
        """
        self.adb_command('adb shell svc wifi enable')

    def p_SERIAL(self, package):
        """
        get p_SERIAL
        :param package: 包名
        :return:p_SERIAL
        """
        rtu = self.adb_command("adb shell \"ps |grep " + package + " |grep -v :\"")
        if rtu == "":
            print(package + " Not have start!")
            return None
        else:
            arr = rtu.split(' ')
            arr = filter(lambda x: x != '', arr)
            return arr[1]

    def get_memory(self, package=None):
        """
        get Memory
        :param package:包名
        :return: 内存对象
        """
        return Memory(self, package)

    def get_focused_package_activity(self):
        """
        获取当前应用界面的包名和Activity
        """
        pa = re.compile(r"[a-zA-Z0-9\.]+/.[a-zA-Z0-9\.]+")
        out = self.adb_command("adb shell dumpsys window w | " + find_util + " \/| " + find_util + " _NAME=")
        if len(pa.findall(out)) < 1:
            return out
        else:
            return pa.findall(out)[0]

    def find_elements(self, _type, value):
        """
        同属性多个元素
        """
        element_list = []
        self.get_layout_xml("LayoutXml.xml")
        tree = ET.ElementTree(file=PATH("LayoutXml.xml"))
        xml_elements = tree.iter(tag="node")
        for xmlElement in xml_elements:
            if xmlElement.attrib[_type] == value:
                bounds = xmlElement.attrib["bounds"]
                pattern = re.compile(r"\d+")
                bound = pattern.findall(bounds)
                x = (int(bound[2]) - int(bound[0])) / 2.0 + int(bound[0])
                y = (int(bound[3]) - int(bound[1])) / 2.0 + int(bound[1])
                # 将匹配的元素区域的中心点添加进pointList中
                element = Element(x, y, self)
                element.TEXT = xmlElement.attrib["text"]
                element.BOUND = bound
                element_list.append(element)
        return element_list

    def find_element(self, _type, value):
        """
        获取元素
        """
        if type is "index_path":
            element = self.find_index_path_element(value)
            return element
        else:
            element = self.find_type_element(_type, value)
            return element

    def find_type_element(self, _type, value):
        """
        根据元素属性定位元素
        """
        self.get_layout_xml("LayoutXml.xml")
        tree = ET.ElementTree(file=PATH("LayoutXml.xml"))
        xml_elements = tree.iter(tag="node")
        element = Element(device=self)
        for xmlElement in xml_elements:
            if xmlElement.attrib[_type] == value:
                bounds = xmlElement.attrib["bounds"]
                pattern = re.compile(r"\d+")
                bound = pattern.findall(bounds)
                element.X_COORDINATE = (int(bound[0]) + int(bound[2])) / 2
                element.Y_COORDINATE = (int(bound[1]) + int(bound[3])) / 2
                element.TEXT = xmlElement.attrib["text"]
                element.BOUND = bound
        return element

    def find_index_path_element(self, value):
        """
        根据xpath定位元素
        """
        self.get_layout_xml("LayoutXml.xml")
        tree = ET.ElementTree(file=PATH("LayoutXml.xml"))
        root = tree.getroot()
        for index in value:
            root = root[index]
        bounds = root.attrib["bounds"]
        pattern = re.compile(r"\d+")
        bound = pattern.findall(bounds)
        x = (int(bound[0]) + int(bound[2])) / 2
        y = (int(bound[1]) + int(bound[3])) / 2
        element = Element(x, y, self)
        element.TEXT = root.attrib["text"]
        element.BOUND = bound
        return element

    def system(self):
        """
        返回系统信息操作对象
        """
        return SystemInfo(self)

    # def get_surface_stats_collector(self):
    #     """
    #     :return SurfaceStatsCollector
    #     """
    #     return self._SurfaceStatsCollector_
    #
    # def fps_stats_start(self, focuse__NAME=None):
    #     """
    #     开始收集fps
    #     :param focuse__NAME:包名None or SurfaceView
    #     """
    #     if focuse__NAME is not None:
    #         self._SurfaceStatsCollector_._focuse__NAME = focuse__NAME
    #     self._SurfaceStatsCollector_.DisableWarningAboutEmptyData()
    #     self._SurfaceStatsCollector_.Start()
    #
    # def fps_stats_stop(self, result__NAME=None):
    #     """
    #     结束收集fps，并返回结果
    #     """
    #     self._SurfaceStatsCollector_.Stop()
    #     results = self._SurfaceStatsCollector_.GetResults()
    #     if result__NAME is not None:
    #         for result in results:
    #             if result._NAME in result__NAME:
    #                 return result
    #         return None
    #     else:
    #         return results

    def clear_log(self):
        """
        清除设备日志缓存
        """
        self.adb_command("adb shell logcat -c")

    def get_log(self, grep=None, f_NAME=None):
        """
        获取设备运行缓存日志
        :param f_NAME:日志保存路径
        :param grep:过滤条件，可以是list或字符串
        """
        cmd_text = "adb shell \" logcat -d"
        if isinstance(grep, list):
            for value in grep:
                cmd_text += " |grep \'" + str(value) + "\'"
        else:
            cmd_text += "|grep " + str(grep)
        cmd_text += " \" "
        if f_NAME is not None:
            cmd_text += "> " + f_NAME
            self.adb_command(cmd_text)
        else:
            return self.adb_command(cmd_text)

# def appLog(self, packge_NAME, Path):
#     P_SERIAL = self.getP_SERIAL(packge_NAME)
#     self.getLog("adb shell \"logcat |grep " + str(P_SERIAL) + "\"", Path)

# def getLog(self, adb_command, Path):
#     GG = adb_commandThread(self, adb_command + " >" + Path)
#     GG.start()
#     # ctypes.pythonapi.PyThreadState_SetAsyncExc(GG._SERIALent, ctypes.py_object(SystemExit))
#     # self.stop_thread(GG)
#     return GG
# def _async_raise(self, t_SERIAL, exctype):
#     """raises the exception, performs cleanup if needed"""
#     t_SERIAL = ctypes.c_long(t_SERIAL)
#     if not inspect.isclass(exctype):
#         exctype = type(exctype)
#     res = ctypes.pythonapi.PyThreadState_SetAsyncExc(t_SERIAL, ctypes.py_object(exctype))
#     if res == 0:
#         raise ValueError("inval_SERIAL thread _SERIAL")
#     elif res != 1:
#         # """if it returns a number greater than one, you're in trouble,
#         # and you should call it again with exc=NULL to revert the effect"""
#         ctypes.pythonapi.PyThreadState_SetAsyncExc(t_SERIAL, None)
#         raise SystemError("PyThreadState_SetAsyncExc failed")

# def stop_thread(self, thread):
#     self._async_raise(thread._SERIALent, SystemExit)

# class adb_commandThread(threading.Thread):  # 继承父类threading.Thread
#     def __init__(self, device, adb_command):
#         threading.Thread.__init__(self)
#         self.device = device
#         self.adb_command = adb_command

#     def run(self):
#         try:
#             print(self.device._SERIAL + "　LogCat...")
#             self.device.adb_command(self.adb_command)
#         except Exception, e:
#             print("Not have connected Device!")



# coding=utf-8
import time

'''
 :Description:    控件类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
'''


class Element(object):
    def __init__(self, x=None, y=None, device=None):
        self.TEXT = None
        self.BOUND = None
        self.X_COORDINATE = x
        self.Y_COORDINATE = y
        self._device_ = device
        self._WIDTH = None
        self._HIGH = None

    def init_screen_size(self):
        self._WIDTH = int(self.BOUND[2]) - int(self.BOUND[0])
        self._HIGH = int(self.BOUND[3]) - int(self.BOUND[1])

    def get_width(self):
        """
        获取元素宽度
        """
        return self._WIDTH

    def get_high(self):
        """
        获取元素高度
        """
        return self._HIGH

    def click(self):
        self._device_.click(self.X_COORDINATE, self.Y_COORDINATE)

    def input(self, text):
        self.click()
        time.sleep(1)
        self._device_.input(text)

    def long_click(self):
        self._device_.longPress(self.X_COORDINATE, self.Y_COORDINATE)

    def swipe_left(self):
        """
        元素向左滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
            self._device_.swipe(int(self.BOUND[2]), int(self.BOUND[1]) + (self._HIGH / 2), int(self.BOUND[0]),
                                int(self.BOUND[1]) + (self._HIGH / 2))
        else:
            self._device_.swipe(int(self.BOUND[2]), int(self.BOUND[1]) + (self._HIGH / 2), int(self.BOUND[0]),
                                int(self.BOUND[1]) + (self._HIGH / 2))

    def swipe_right(self):
        """
        元素向右滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
            self._device_.swipe(int(self.BOUND[0]), self.BOUND[1] + (self._HIGH / 2), int(self.BOUND[2]),
                                int(self.BOUND[1]) + (self._HIGH / 2))
        else:
            self._device_.swipe(int(self.BOUND[0]), self.BOUND[1] + (self._HIGH / 2), int(self.BOUND[2]),
                                int(self.BOUND[1]) + (self._HIGH / 2))

    def swipe_up(self):
        """
        元素向上滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
            self._device_.swipe(int(self.BOUND[2]) + (self._WIDTH / 2), int(self.BOUND[3]),
                                int(self.BOUND[2]) + (self._WIDTH / 2),
                                int(self.BOUND[1]))
        else:
            self._device_.swipe(int(self.BOUND[2]) + (self._WIDTH / 2), int(self.BOUND[3]),
                                int(self.BOUND[2]) + (self._WIDTH / 2),
                                int(self.BOUND[1]))

    def swipe_down(self):
        """
        元素向下滑动
        """
        if self._WIDTH is None or self._HIGH is None:
            self.init_screen_size()
            self._device_.swipe(int(self.BOUND[2]) + (self._WIDTH / 2), int(self.BOUND[1]),
                                int(self.BOUND[2]) + (self._WIDTH / 2),
                                int(self.BOUND[3]))
        else:
            self._device_.swipe(int(self.BOUND[2]) + (self._WIDTH / 2), int(self.BOUND[1]),
                                int(self.BOUND[2]) + (self._WIDTH / 2),
                                int(self.BOUND[3]))
